/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.xerces.utils;

import java.lang.*;


/**
 * format validation
 *
 * This class encodes/decodes hexadecimal data
 * @author Jeffrey Rodriguez
 * @version $Id: HexBin.java,v 1.4 2000/11/29 00:22:23 jeffreyr Exp $
 */

public final class  HexBin {
    static private final int  BASELENGTH   = 255;       
    static private final int  LOOKUPLENGTH = 16;
    static private byte [] hexNumberTable       = new byte[BASELENGTH]; 
    static private byte [] lookUpHexAlphabet = new byte[LOOKUPLENGTH];


    static {
        for (int i = 0; i<BASELENGTH; i++ ) {  
            hexNumberTable[i] = -1; 
        }
        for ( int i = '9'; i >= '0'; i--) {
            hexNumberTable[i] = (byte) (i-'0');
        }
        for ( int i = 'F'; i>= 'A'; i--) {
            hexNumberTable[i] = (byte) ( i-'A' + 10 );
        }
        for ( int i = 'f'; i>= 'a'; i--) {
           hexNumberTable[i] = (byte) ( i-'a' + 10 );
        }

        for(int i = 0; i<10; i++ )
            lookUpHexAlphabet[i] = (byte) ('0'+i );
        for(int i = 10; i<=15; i++ )
            lookUpHexAlphabet[i] = (byte) ('A'+i -10);
    }

    /**
     * byte to be tested if it is Base64 alphabet
     * 
     * @param octect
     * @return 
     */
    static boolean isHex( byte octect ) {
        //shall we ignore white space? JEFF??
        return(hexNumberTable[octect] != -1 ); 
    }


    /**
     *       Array of bytes to check against Hex Table 
     * 
     * @param arrayOctect
     * @return 
     */
    static boolean isArrayByteHex( byte[] arrayOctect ) {
       int length = arrayOctect.length;
       if( length == 0 )
           return false;
       for( int i=0; i < length; i++ ){   
           
           if( HexBin.isHex( arrayOctect[i] ) == false)
               return false;

       }
       return true;
   }

    public static boolean isHex( String isValidString ){
      return( isArrayByteHex( isValidString.getBytes()));
  }



    /**
     * array of byte to encode
     * 
     * @param binaryData
     * @return return encode binary array
     */
    static public byte[] encode( byte[] binaryData ) {
      int lengthData   = binaryData.length;
      int lengthEncode = lengthData;
      byte[] encodedData = new byte[lengthData];
      for( int i = 0; i<lengthData; i++ ){
         encodedData[i] = lookUpHexAlphabet[ binaryData[i] ];
      }
      return encodedData;
    }


    static public  byte[] decode ( byte[]  binaryData ) {
      int lengthData   = binaryData.length;
      int lengthEncode = lengthData;
      byte[] decodedData = new byte[lengthData];
      for( int i = 0; i<lengthData; i++ ){
      decodedData[i] = hexNumberTable[binaryData[i]];
      }
     return decodedData;
    }
}
